from .format import decide_format
from flask import request, make_response


def _get_graph(output):
	""" Given a Flask response, find the rdflib Graph """
	if hasattr(output, 'serialize'):	# single graph object
		return output

	if hasattr(output, '__getitem__'):	# indexable tuple
		if len(output) > 0 and \
		   hasattr(output[0], 'serialize'):	# graph object
			return output[0]


def _set_graph(output, newgraph):
	""" Replace the rdflib Graph in a Flask response """
	if hasattr(output, 'serialize'):	# single graph object
		return newgraph

	if hasattr(output, '__getitem__'):	# indexable tuple
		if len(output) > 0 and \
		   hasattr(output[0], 'serialize'):	# graph object
			return (newgraph,) + output[1:]

def output_flask(output, accepts):
	""" Formats a response from a Flask view to handle any RDF graphs
	    If a view function returns a single RDF graph, serialize it based on Accept header
	    If it's an RDF graph plus some extra headers, pass those along
	    If it's not an RDF graph at all, return it without any special handling
	"""

	graph = _get_graph(output)
	if graph is not None:
		# decide the format
		output_mimetype, output_format = decide_format(accepts, graph.context_aware)
		if 'text' in output_mimetype:
			output_mimetype = output_mimetype + '; charset=utf-8'

		# format the new response
		serialized = graph.serialize(format=output_format)
		final_output = _set_graph(output, serialized)
		response = make_response(final_output)
		response.headers['Content-Type'] = output_mimetype
		return response
	else:
		return make_response(output)


def flask_rdf(view):
	""" Wraps a Flask view function to return formatted RDF graphs
	    Uses content negotiation to serialize the graph to the client-preferred format
	    Passes other content through unmodified
	"""
	from flask import request, make_response
	from functools import wraps

	@wraps(view)
	def decorated(*args, **kwargs):
		output = view(*args, **kwargs)
		return output_flask(output, request.headers.get('Accept', ''))
	return decorated



import mimeparse


# What formats we support for serialization
formats = {
   'application/x-turtle': 'turtle',
   'text/turtle': 'turtle',
   'application/rdf+xml': 'xml',
   'application/trix': 'trix',
   'application/n-quads': 'nquads',
   'application/n-triples': 'nt',
   'text/n-triples': 'nt',
   'text/rdf+nt': 'nt',
   'application/n3': 'n3',
   'text/n3': 'n3',
   'text/rdf+n3': 'n3'
}
# the list of any mimetypes, unlocked if we have a context
all_mimetypes = list(formats.keys())
# the list of mimetypes that don't require a context
ctxless_mimetypes = [m for m in all_mimetypes if 'n-quads' not in m]


def add_format(mimetype, format, requires_context=False):
	""" Registers a new format to be used in a graph's serialize call
	    If you've installed an rdflib serializer plugin, use this
	    to add it to the content negotiation system
	    Set requires_context=True if this format requires a context-aware graph
	"""
	global formats
	formats[mimetype] = format
	if not requires_context:
		ctxless_mimetypes.append(mimetype)
	all_mimetypes.append(mimetype)


def decide_format(accepts, context_aware = False):
	""" Returns what (mimetype,format) the client wants to receive
	    Parses the given Accept header and picks the best one that
	    we know how to output
	    Returns (mimetype, format)
	    An unknown Accept will default to rdf+xml
	    context_aware=True will allow nquad serialization
	"""
	if context_aware:
		mimetype = mimeparse.best_match(all_mimetypes, accepts)
	else:
		mimetype = mimeparse.best_match(ctxless_mimetypes, accepts)
	if mimetype:
		return (mimetype, formats[mimetype])
	else:
		return ('application/rdf+xml', 'xml')



#!/usr/bin/env python

from distutils.core import setup
try:
	from setuptools import setup
except:
	pass

requirements = ['Flask', 'python-mimeparse==0.1.4']
test_requirements = ['nose==1.3.3', 'rdflib']

long_description = open('README.rst').read()

setup(name='flask_rdf',
      version='0.1.8',
      description='Flask decorator to output RDF using content negotiation',
      author='Walter Huf',
      author_email='hufman@gmail.com',
      url='https://github.com/hufman/flask_rdf',
      packages=['flask_rdf'],
      install_requires=requirements,
      test_requires=requirements + test_requirements,
      long_description=long_description,
      classifiers=[
          'Development Status :: 4 - Beta',
          'Framework :: Flask',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: BSD License',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Topic :: Internet :: WWW/HTTP :: Dynamic Content :: CGI Tools/Libraries',
          'Topic :: Software Development :: Libraries :: Python Modules'
     ],
     license='BSD',
     platforms=['Any']
)
